@@ -16,12 +16,15 @@ private = re.compile("_[^_]")
 
 def _run_until_done(coro):
     """execute coroutine, when already in the event loop"""
+    if sys.version_info < (3, 7):  # pragma: no cover
+        raise RuntimeError(
+            "async file systems do not work completely on py<37. "
+            "The nested call currently underway cannot be processed. "
+            "Please downgrade your fsspec or upgrade python."
+        )
     loop = asyncio.get_event_loop()
-    assert loop.is_running()
     task = asyncio.current_task()
     asyncio.tasks._unregister_task(task)
-    current_task = asyncio.tasks._current_tasks.get(loop)
-    assert task == current_task
     del asyncio.tasks._current_tasks[loop]
     runner = loop.create_task(coro)
     while not runner.done():
@@ -193,14 +196,25 @@ class AsyncFileSystem(AbstractFileSystem):
         maybe_sync(self._rm, self, path, **kwargs)
 
     async def _copy(self, paths, path2, **kwargs):
-        await asyncio.gather(
-            *[self._cp_file(p1, p2, **kwargs) for p1, p2 in zip(paths, path2)]
+        return await asyncio.gather(
+            *[self._cp_file(p1, p2, **kwargs) for p1, p2 in zip(paths, path2)],
+            return_exceptions=True
         )
 
-    def copy(self, path1, path2, recursive=False, **kwargs):
+    def copy(self, path1, path2, recursive=False, on_error=None, **kwargs):
+        if on_error is None and recursive:
+            on_error = "ignore"
+        elif on_error is None:
+            on_error = "raise"
+
         paths = self.expand_path(path1, recursive=recursive)
         path2 = other_paths(paths, path2)
-        maybe_sync(self._copy, self, paths, path2)
+        result = maybe_sync(self._copy, self, paths, path2, **kwargs)
+
+        for ex in filter(is_exception, result):
+            if on_error == "ignore" and isinstance(ex, FileNotFoundError):
+                continue
+            raise ex
 
     async def _pipe(self, path, value=None, **kwargs):
         if isinstance(path, str):
