@@ -1,10 +1,10 @@
-import time
 import pickle
 import logging
 import os
 import hashlib
 from shutil import move, rmtree
 import tempfile
+import time
 import inspect
 
 from fsspec import AbstractFileSystem, filesystem
@@ -105,6 +105,8 @@ class CachingFileSystem(AbstractFileSystem):
         self.check_files = check_files
         self.expiry = expiry_time
         self.compression = compression
+        # TODO: same_names should allow for variable prefix, not only
+        #  to keep the basename
         self.same_names = same_names
         self.target_protocol = (
             target_protocol
@@ -152,11 +154,12 @@ class CachingFileSystem(AbstractFileSystem):
             with open(fn, "rb") as f:
                 cached_files = pickle.load(f)
             for k, c in cached_files.items():
-                if c["blocks"] is not True:
-                    if cache[k]["blocks"] is True:
+                if k in cache:
+                    if c["blocks"] is True or cache[k]["blocks"] is True:
                         c["blocks"] = True
                     else:
                         c["blocks"] = set(c["blocks"]).union(cache[k]["blocks"])
+                    c["time"] = max(c["time"], cache[k]["time"])
 
             # Files can be added to cache after it was written once
             for k, c in cache.items():
@@ -173,6 +176,8 @@ class CachingFileSystem(AbstractFileSystem):
             pickle.dump(cache, f)
         self._mkcache()
         move(fn2, fn)
+        self.cached_files[-1] = cached_files
+        self.last_cache = time.time()
 
     def _check_cache(self):
         """Reload caches if time elapsed or any disappeared"""
@@ -190,20 +195,15 @@ class CachingFileSystem(AbstractFileSystem):
         path = self._strip_protocol(path)
 
         self._check_cache()
-        if not path.startswith(self.target_protocol):
-            store_path = self.target_protocol + "://" + path
-            path = self.fs._strip_protocol(store_path)
-        else:
-            store_path = path
         for storage, cache in zip(self.storage, self.cached_files):
-            if store_path not in cache:
+            if path not in cache:
                 continue
-            detail = cache[store_path].copy()
+            detail = cache[path].copy()
             if self.check_files:
                 if detail["uid"] != self.fs.ukey(path):
                     continue
             if self.expiry:
-                if detail["time"] - time.time() > self.expiry:
+                if time.time() - detail["time"] > self.expiry:
                     continue
             fn = os.path.join(storage, detail["fn"])
             if os.path.exists(fn):
@@ -227,18 +227,13 @@ class CachingFileSystem(AbstractFileSystem):
         raises PermissionError
         """
         path = self._strip_protocol(path)
-        if not path.startswith(self.target_protocol):
-            store_path = self.target_protocol + "://" + path
-            path = self.fs._strip_protocol(store_path)
-        else:
-            store_path = path
         _, fn = self._check_file(path)
         if fn is None:
             return
         if fn.startswith(self.storage[-1]):
             # is in in writable cache
             os.remove(fn)
-            self.cached_files[-1].pop(store_path)
+            self.cached_files[-1].pop(path)
             self.save_cache()
         else:
             raise PermissionError(
@@ -268,11 +263,7 @@ class CachingFileSystem(AbstractFileSystem):
         """
         path = self._strip_protocol(path)
 
-        if not path.startswith(self.target_protocol):
-            store_path = self.target_protocol + "://" + path
-        else:
-            store_path = path
-        path = self.fs._strip_protocol(store_path)
+        path = self.fs._strip_protocol(path)
         if "r" not in mode:
             return self.fs._open(
                 path,
@@ -282,7 +273,7 @@ class CachingFileSystem(AbstractFileSystem):
                 cache_options=cache_options,
                 **kwargs
             )
-        detail = self._check_file(store_path)
+        detail = self._check_file(path)
         if detail:
             # file is in cache
             detail, fn = detail
@@ -303,7 +294,7 @@ class CachingFileSystem(AbstractFileSystem):
                 "time": time.time(),
                 "uid": self.fs.ukey(path),
             }
-            self.cached_files[-1][store_path] = detail
+            self.cached_files[-1][path] = detail
             logger.debug("Creating local sparse file for %s" % path)
 
         # call target filesystems open
@@ -344,14 +335,21 @@ class CachingFileSystem(AbstractFileSystem):
 
     def close_and_update(self, f, close):
         """Called when a file is closing, so store the set of blocks"""
+        if f.closed:
+            return
         path = self._strip_protocol(f.path)
 
-        if not path.startswith(self.target_protocol):
-            store_path = self.target_protocol + "://" + path
-        c = self.cached_files[-1][store_path]
+        c = self.cached_files[-1][path]
         if c["blocks"] is not True and len(["blocks"]) * f.blocksize >= f.size:
             c["blocks"] = True
-        self.save_cache()
+        try:
+            logger.debug("going to save")
+            self.save_cache()
+            logger.debug("saved")
+        except (IOError, OSError):
+            logger.debug("Cache saving failed while closing file")
+        except NameError:
+            logger.debug("Cache save failed due to interpreter shutdown")
         close()
 
     def __getattribute__(self, item):
@@ -363,6 +361,7 @@ class CachingFileSystem(AbstractFileSystem):
             "__init__",
             "__getattribute__",
             "__reduce__",
+            "_make_local_details",
             "open",
             "cat",
             "cat_file",
@@ -372,6 +371,7 @@ class CachingFileSystem(AbstractFileSystem):
             "head",
             "_check_file",
             "_check_cache",
+            "_mkcache",
             "clear_cache",
             "pop_from_cache",
             "_mkcache",
@@ -431,7 +431,7 @@ class WholeFileCacheFileSystem(CachingFileSystem):
     local_file = True
 
     def open_many(self, open_files):
-        paths, store_paths = zip(*[self._paths_from_path(of.path) for of in open_files])
+        paths = [of.path for of in open_files]
         if "r" in open_files.mode:
             self._mkcache()
         else:
@@ -442,9 +442,8 @@ class WholeFileCacheFileSystem(CachingFileSystem):
 
         if self.compression:
             raise NotImplementedError
-        details = [self._check_file(sp) for sp in store_paths]
+        details = [self._check_file(sp) for sp in paths]
         downpath = [p for p, d in zip(paths, details) if not d]
-        downstore = [p for p, d in zip(store_paths, details) if not d]
         downfn0 = [
             os.path.join(self.storage[-1], self.hash_name(p, self.same_names))
             for p, d in zip(paths, details)
@@ -465,7 +464,7 @@ class WholeFileCacheFileSystem(CachingFileSystem):
                 for path in downpath
             ]
             self.cached_files[-1].update(
-                {store_path: detail for store_path, detail in zip(downstore, newdetail)}
+                {path: detail for path, detail in zip(downpath, newdetail)}
             )
             self.save_cache()
 
@@ -481,24 +480,51 @@ class WholeFileCacheFileSystem(CachingFileSystem):
     def commit_many(self, open_files):
         self.fs.put([f.fn for f in open_files], [f.path for f in open_files])
 
-    def _paths_from_path(self, path):
-        path = self._strip_protocol(path)
+    def _make_local_details(self, path):
+        hash = self.hash_name(path, self.same_names)
+        fn = os.path.join(self.storage[-1], hash)
+        detail = {
+            "fn": hash,
+            "blocks": True,
+            "time": time.time(),
+            "uid": self.fs.ukey(path),
+        }
+        self.cached_files[-1][path] = detail
+        logger.debug("Copying %s to local cache" % path)
+        return fn
 
-        if not path.startswith(self.target_protocol):
-            store_path = self.target_protocol + "://" + path
-        else:
-            store_path = path
-        path = self.fs._strip_protocol(store_path)
-        return path, store_path
+    def cat(self, path, recursive=False, on_error="raise", **kwargs):
+        paths = self.expand_path(
+            path, recursive=recursive, maxdepth=kwargs.get("maxdepth", None)
+        )
+        getpaths = []
+        storepaths = []
+        fns = []
+        for p in paths:
+            detail = self._check_file(p)
+            if not detail:
+                fn = self._make_local_details(p)
+                getpaths.append(p)
+                storepaths.append(fn)
+            else:
+                detail, fn = detail if isinstance(detail, tuple) else (None, detail)
+            fns.append(fn)
+        if getpaths:
+            self.fs.get(getpaths, storepaths)
+            self.save_cache()
+        out = {path: open(fn, "rb").read() for path, fn in zip(paths, fns)}
+        if isinstance(path, str) and len(paths) == 1 and recursive is False:
+            out = out[paths[0]]
+        return out
 
     def _open(self, path, mode="rb", **kwargs):
-        path, store_path = self._paths_from_path(path)
+        path = self._strip_protocol(path)
         if "r" not in mode:
             return self.fs._open(path, mode=mode, **kwargs)
-        detail = self._check_file(store_path)
+        detail = self._check_file(path)
         if detail:
             detail, fn = detail
-            hash, blocks = detail["fn"], detail["blocks"]
+            _, blocks = detail["fn"], detail["blocks"]
             if blocks is True:
                 logger.debug("Opening local copy of %s" % path)
                 return open(fn, mode)
@@ -508,43 +534,29 @@ class WholeFileCacheFileSystem(CachingFileSystem):
                     "as a wholly cached file" % path
                 )
         else:
-            hash = self.hash_name(path, self.same_names)
-            fn = os.path.join(self.storage[-1], hash)
-            detail = {
-                "fn": hash,
-                "blocks": True,
-                "time": time.time(),
-                "uid": self.fs.ukey(path),
-            }
-            self.cached_files[-1][store_path] = detail
-            logger.debug("Copying %s to local cache" % path)
+            fn = self._make_local_details(path)
         kwargs["mode"] = mode
 
         # call target filesystems open
-        # TODO: why not just use fs.get ??
-        f = self.fs._open(path, **kwargs)
-        if self.compression:
-            comp = (
-                infer_compression(path)
-                if self.compression == "infer"
-                else self.compression
-            )
-            f = compr[comp](f, mode="rb")
         self._mkcache()
-        with open(fn, "wb") as f2:
-            if isinstance(f, AbstractBufferedFile):
-                # want no type of caching if just downloading whole thing
-                f.cache = BaseCache(0, f.cache.fetcher, f.size)
-            if getattr(f, "blocksize", 0) and f.size:
-                # opportunity to parallelise here
+        if self.compression:
+            with self.fs._open(path, **kwargs) as f, open(fn, "wb") as f2:
+                if isinstance(f, AbstractBufferedFile):
+                    # want no type of caching if just downloading whole thing
+                    f.cache = BaseCache(0, f.cache.fetcher, f.size)
+                comp = (
+                    infer_compression(path)
+                    if self.compression == "infer"
+                    else self.compression
+                )
+                f = compr[comp](f, mode="rb")
                 data = True
-                block = getattr(f, "blocksize", 5 * 2 ** 20)
                 while data:
+                    block = getattr(f, "blocksize", 5 * 2 ** 20)
                     data = f.read(block)
                     f2.write(data)
-            else:
-                # this only applies to HTTP, should instead use streaming
-                f2.write(f.read())
+        else:
+            self.fs.get(path, fn)
         self.save_cache()
         return self._open(path, mode)
 
@@ -595,11 +607,6 @@ class SimpleCacheFileSystem(WholeFileCacheFileSystem):
     def _open(self, path, mode="rb", **kwargs):
         path = self._strip_protocol(path)
 
-        if not path.startswith(self.target_protocol):
-            store_path = self.target_protocol + "://" + path
-        else:
-            store_path = path
-        path = self.fs._strip_protocol(store_path)
         if "r" not in mode:
             return LocalTempFile(self, path, mode=mode)
         fn = self._check_file(path)
