@@ -46,6 +46,8 @@ class FTPFileSystem(AbstractFileSystem):
             If given, the read-ahead or write buffer size.
         tempdir: str
             Directory on remote to put temporary files when in a transaction
+        timeout: int
+            Timeout of the ftp connection in seconds
         """
         super(FTPFileSystem, self).__init__(**kwargs)
         self.host = host
@@ -146,15 +148,17 @@ class FTPFileSystem(AbstractFileSystem):
     def _rm(self, path):
         path = self._strip_protocol(path)
         self.ftp.delete(path)
-        self.invalidate_cache(path.rsplit("/", 1)[0])
+        self.invalidate_cache(self._parent(path))
 
     def mkdir(self, path, **kwargs):
         path = self._strip_protocol(path)
         self.ftp.mkd(path)
+        self.invalidate_cache(self._parent(path))
 
     def rmdir(self, path):
         path = self._strip_protocol(path)
         self.ftp.rmd(path)
+        self.invalidate_cache(self._parent(path))
 
     def mv(self, path1, path2, **kwargs):
         path1 = self._strip_protocol(path1)
@@ -166,6 +170,13 @@ class FTPFileSystem(AbstractFileSystem):
     def __del__(self):
         self.ftp.close()
 
+    def invalidate_cache(self, path=None):
+        if path is None:
+            self.dircache.clear()
+        else:
+            self.dircache.pop(path, None)
+        super(FTPFileSystem, self).invalidate_cache(path)
+
 
 class TransferDone(Exception):
     """Internal exception to break out of transfer"""
