@@ -95,10 +95,17 @@ class HTTPFileSystem(AsyncFileSystem):
 
     @classmethod
     def _strip_protocol(cls, path):
-        """ For HTTP, we always want to keep the full URL
-        """
+        """For HTTP, we always want to keep the full URL"""
         return path
 
+    @classmethod
+    def _parent(cls, path):
+        # override, since _strip_protocol is different for URLs
+        par = super()._parent(path)
+        if len(par) > 7:  # "http://..."
+            return par
+        return ""
+
     async def _ls(self, url, detail=True, **kwargs):
         # ignoring URL-encoded arguments
         kw = self.kwargs.copy()
@@ -145,10 +152,16 @@ class HTTPFileSystem(AsyncFileSystem):
         else:
             return list(sorted(out))
 
-    async def _cat_file(self, url, **kwargs):
+    async def _cat_file(self, url, start=None, end=None, **kwargs):
         kw = self.kwargs.copy()
         kw.update(kwargs)
         logger.debug(url)
+        if (start is None) ^ (end is None):
+            raise ValueError("Give start and end or neither")
+        if start is not None:
+            headers = kw.pop("headers", {}).copy()
+            headers["Range"] = "bytes=%i-%i" % (start, end - 1)
+            kw["headers"] = headers
         async with self.session.get(url, **kw) as r:
             if r.status == 404:
                 raise FileNotFoundError(url)
@@ -178,7 +191,7 @@ class HTTPFileSystem(AsyncFileSystem):
             r = await self.session.get(path, **kw)
             async with r:
                 return r.status < 400
-        except requests.HTTPError:
+        except (requests.HTTPError, aiohttp.client_exceptions.ClientError):
             return False
 
     async def _isfile(self, path, **kwargs):
@@ -192,6 +205,7 @@ class HTTPFileSystem(AsyncFileSystem):
         autocommit=None,  # XXX: This differs from the base class.
         cache_type=None,
         cache_options=None,
+        size=None,
         **kwargs
     ):
         """Make a file-like object
@@ -214,7 +228,7 @@ class HTTPFileSystem(AsyncFileSystem):
         kw = self.kwargs.copy()
         kw["asynchronous"] = self.asynchronous
         kw.update(kwargs)
-        size = self.size(path)
+        size = size or self.size(path)
         if block_size and size:
             return HTTPFile(
                 self,
@@ -263,6 +277,84 @@ class HTTPFileSystem(AsyncFileSystem):
                 raise FileNotFoundError(url)
         return {"name": url, "size": size or None, "type": "file"}
 
+    def glob(self, path, **kwargs):
+        """
+        Find files by glob-matching.
+
+        This implementation is idntical to the one in AbstractFileSystem,
+        but "?" is not considered as a character for globbing, because it is
+        so common in URLs, often identifying the "query" part.
+        """
+        import re
+
+        ends = path.endswith("/")
+        path = self._strip_protocol(path)
+        indstar = path.find("*") if path.find("*") >= 0 else len(path)
+        indbrace = path.find("[") if path.find("[") >= 0 else len(path)
+
+        ind = min(indstar, indbrace)
+
+        detail = kwargs.pop("detail", False)
+
+        if not has_magic(path):
+            root = path
+            depth = 1
+            if ends:
+                path += "/*"
+            elif self.exists(path):
+                if not detail:
+                    return [path]
+                else:
+                    return {path: self.info(path)}
+            else:
+                if not detail:
+                    return []  # glob of non-existent returns empty
+                else:
+                    return {}
+        elif "/" in path[:ind]:
+            ind2 = path[:ind].rindex("/")
+            root = path[: ind2 + 1]
+            depth = None if "**" in path else path[ind2 + 1 :].count("/") + 1
+        else:
+            root = ""
+            depth = None if "**" in path else path[ind + 1 :].count("/") + 1
+
+        allpaths = self.find(root, maxdepth=depth, withdirs=True, detail=True, **kwargs)
+        # Escape characters special to python regex, leaving our supported
+        # special characters in place.
+        # See https://www.gnu.org/software/bash/manual/html_node/Pattern-Matching.html
+        # for shell globbing details.
+        pattern = (
+            "^"
+            + (
+                path.replace("\\", r"\\")
+                .replace(".", r"\.")
+                .replace("+", r"\+")
+                .replace("//", "/")
+                .replace("(", r"\(")
+                .replace(")", r"\)")
+                .replace("|", r"\|")
+                .replace("^", r"\^")
+                .replace("$", r"\$")
+                .replace("{", r"\{")
+                .replace("}", r"\}")
+                .rstrip("/")
+            )
+            + "$"
+        )
+        pattern = re.sub("[*]{2}", "=PLACEHOLDER=", pattern)
+        pattern = re.sub("[*]", "[^/]*", pattern)
+        pattern = re.compile(pattern.replace("=PLACEHOLDER=", ".*"))
+        out = {
+            p: allpaths[p]
+            for p in sorted(allpaths)
+            if pattern.match(p.replace("//", "/").rstrip("/"))
+        }
+        if detail:
+            return out
+        else:
+            return list(out)
+
     def isdir(self, path):
         # override, since all URLs are (also) files
         return bool(self.ls(path))
@@ -360,7 +452,9 @@ class HTTPFile(AbstractBufferedFile):
             async with r:
                 r.raise_for_status()
                 out = await r.read()
-                self.cache = AllBytes(out)
+                self.cache = AllBytes(
+                    size=len(out), fetcher=None, blocksize=None, data=out
+                )
                 self.size = len(out)
 
     _fetch_all = sync_wrapper(async_fetch_all)
@@ -419,9 +513,29 @@ class HTTPFile(AbstractBufferedFile):
     def close(self):
         pass
 
+    def __reduce__(self):
+        return reopen, (
+            self.fs,
+            self.url,
+            self.mode,
+            self.blocksize,
+            self.cache.name,
+            self.size,
+        )
 
-async def get(session, url, **kwargs):
-    return await session.get(url, **kwargs)
+
+def reopen(fs, url, mode, blocksize, cache_type, size=None):
+    return fs.open(
+        url, mode=mode, block_size=blocksize, cache_type=cache_type, size=size
+    )
+
+
+magic_check = re.compile("([*[])")
+
+
+def has_magic(s):
+    match = magic_check.search(s)
+    return match is not None
 
 
 class HTTPStreamFile(AbstractBufferedFile):
@@ -434,10 +548,10 @@ class HTTPStreamFile(AbstractBufferedFile):
             raise ValueError
         self.details = {"name": url, "size": None}
         super().__init__(fs=fs, path=url, mode=mode, cache_type="none", **kwargs)
-        self.r = sync(self.loop, get, self.session, url, **kwargs)
+        self.r = sync(self.loop, self.session.get, url, **kwargs)
 
     def seek(self, *args, **kwargs):
-        raise ValueError("Cannot seek strteaming HTTP file")
+        raise ValueError("Cannot seek streaming HTTP file")
 
     async def _read(self, num=-1):
         out = await self.r.content.read(num)
@@ -452,6 +566,9 @@ class HTTPStreamFile(AbstractBufferedFile):
     def close(self):
         asyncio.run_coroutine_threadsafe(self._close(), self.loop)
 
+    def __reduce__(self):
+        return reopen, (self.fs, self.url, self.mode, self.blocksize, self.cache.name)
+
 
 async def get_range(session, url, start, end, file=None, **kwargs):
     # explicit get a range when we know it must be safe
