@@ -22,6 +22,7 @@ class LocalFileSystem(AbstractFileSystem):
 
     root_marker = "/"
     protocol = "file"
+    local_file = True
 
     def __init__(self, auto_mkdir=False, **kwargs):
         super().__init__(**kwargs)
@@ -39,9 +40,10 @@ class LocalFileSystem(AbstractFileSystem):
         os.makedirs(path, exist_ok=exist_ok)
 
     def rmdir(self, path):
+        path = self._strip_protocol(path)
         os.rmdir(path)
 
-    def ls(self, path, detail=False):
+    def ls(self, path, detail=False, **kwargs):
         path = self._strip_protocol(path)
         paths = [posixpath.join(path, f) for f in os.listdir(path)]
         if detail:
@@ -49,9 +51,9 @@ class LocalFileSystem(AbstractFileSystem):
         else:
             return paths
 
-    def glob(self, path, **kargs):
+    def glob(self, path, **kwargs):
         path = self._strip_protocol(path)
-        return super().glob(path)
+        return super().glob(path, **kwargs)
 
     def info(self, path, **kwargs):
         path = self._strip_protocol(path)
@@ -78,25 +80,31 @@ class LocalFileSystem(AbstractFileSystem):
                 result["size"] = 0
         return result
 
-    def copy(self, path1, path2, **kwargs):
-        shutil.copyfile(path1, path2)
-
-    def get(self, path1, path2, **kwargs):
-        if kwargs.get("recursive"):
-            return super(LocalFileSystem, self).get(path1, path2, **kwargs)
+    def cp_file(self, path1, path2, **kwargs):
+        path1 = self._strip_protocol(path1).rstrip("/")
+        path2 = self._strip_protocol(path2).rstrip("/")
+        if self.auto_mkdir:
+            self.makedirs(self._parent(path2), exist_ok=True)
+        if self.isfile(path1):
+            shutil.copyfile(path1, path2)
+        elif self.isdir(path1):
+            self.mkdirs(path2, exist_ok=True)
         else:
-            return self.copy(path1, path2, **kwargs)
+            raise FileNotFoundError
 
-    def put(self, path1, path2, **kwargs):
-        if kwargs.get("recursive"):
-            return super(LocalFileSystem, self).put(path1, path2, **kwargs)
-        else:
-            return self.copy(path1, path2, **kwargs)
+    def get_file(self, path1, path2, **kwargs):
+        return self.cp_file(path1, path2, **kwargs)
 
-    def mv(self, path1, path2, **kwargs):
+    def put_file(self, path1, path2, **kwargs):
+        return self.cp_file(path1, path2, **kwargs)
+
+    def mv_file(self, path1, path2, **kwargs):
+        path1 = self._strip_protocol(path1).rstrip("/")
+        path2 = self._strip_protocol(path2).rstrip("/")
         os.rename(path1, path2)
 
     def rm(self, path, recursive=False, maxdepth=None):
+        path = self._strip_protocol(path).rstrip("/")
         if recursive and self.isdir(path):
             shutil.rmtree(path)
         else:
@@ -110,6 +118,8 @@ class LocalFileSystem(AbstractFileSystem):
 
     def touch(self, path, **kwargs):
         path = self._strip_protocol(path)
+        if self.auto_mkdir:
+            self.makedirs(self._parent(path), exist_ok=True)
         if self.exists(path):
             os.utime(path, None)
         else:
@@ -148,6 +158,8 @@ class LocalFileSystem(AbstractFileSystem):
 
 def make_path_posix(path, sep=os.sep):
     """ Make path generic """
+    if isinstance(path, (list, set, tuple)):
+        return type(path)(make_path_posix(p) for p in path)
     if re.match("/[A-Za-z]:", path):
         # for windows file URI like "file:///C:/folder/file"
         # or "file:///C:\\dir\\file"
@@ -210,13 +222,13 @@ class LocalFileOpener(object):
         return self.f.read(end - start)
 
     def __setstate__(self, state):
+        self.f = None
+        loc = state.pop("loc", None)
+        self.__dict__.update(state)
         if "r" in state["mode"]:
-            loc = self.state.pop("loc")
+            self.f = None
             self._open()
             self.f.seek(loc)
-        else:
-            self.f = None
-        self.__dict__.update(state)
 
     def __getstate__(self):
         d = self.__dict__.copy()
@@ -242,6 +254,9 @@ class LocalFileOpener(object):
         # uniquely among fsspec implementations, this is a real, local path
         return self.path
 
+    def __iter__(self):
+        return self.f.__iter__()
+
     def __getattr__(self, item):
         return getattr(self.f, item)
 
