@@ -1,6 +1,8 @@
 from __future__ import print_function, division, absolute_import
 
 from io import BytesIO
+from datetime import datetime
+from errno import ENOTEMPTY
 from fsspec import AbstractFileSystem
 import logging
 
@@ -8,7 +10,11 @@ logger = logging.Logger("fsspec.memoryfs")
 
 
 class MemoryFileSystem(AbstractFileSystem):
-    """A filesystem based on a dict of BytesIO objects"""
+    """A filesystem based on a dict of BytesIO objects
+
+    This is a global filesystem so instances of this class all point to the same
+    in memory filesystem.
+    """
 
     store = {}  # global
     pseudo_dirs = []
@@ -23,6 +29,7 @@ class MemoryFileSystem(AbstractFileSystem):
                     "name": path,
                     "size": self.store[path].getbuffer().nbytes,
                     "type": "file",
+                    "created": self.store[path].created,
                 }
             ]
         else:
@@ -42,10 +49,15 @@ class MemoryFileSystem(AbstractFileSystem):
                         "name": has_slash + p,
                         "size": self.store[p2].getbuffer().nbytes,
                         "type": "file",
+                        "created": self.store[p2].created,
                     }
                 )
-            elif path and all(
-                (a == b) for a, b in zip(path.split("/"), p.strip("/").split("/"))
+            elif (
+                path
+                and len(path) < len(p.strip("/"))
+                and all(
+                    (a == b) for a, b in zip(path.split("/"), p.strip("/").split("/"))
+                )
             ):
                 # implicit directory
                 ppath = "/".join(p.split("/")[: len(path.split("/")) + 1])
@@ -74,7 +86,7 @@ class MemoryFileSystem(AbstractFileSystem):
                     )
                     paths.add(ppath)
         for p2 in self.pseudo_dirs:
-            if self._parent(p2).strip("/").rstrip("/") == path:
+            if self._parent(p2).strip("/") == path and p2.strip("/") not in paths:
                 out.append({"name": p2 + "/", "size": 0, "type": "directory"})
         if detail:
             return out
@@ -95,7 +107,7 @@ class MemoryFileSystem(AbstractFileSystem):
             if not self.ls(path):
                 self.pseudo_dirs.remove(path)
             else:
-                raise OSError("Directory %s not empty" % path)
+                raise OSError(ENOTEMPTY, "Directory not empty", path)
         else:
             raise FileNotFoundError(path)
 
@@ -152,6 +164,17 @@ class MemoryFileSystem(AbstractFileSystem):
         else:
             raise FileNotFoundError
 
+    def rm(self, path, recursive=False, maxdepth=None):
+        paths = self.expand_path(path, recursive=recursive, maxdepth=maxdepth)
+        for p in reversed(paths):
+            # If the expanded path doesn't exist, it is only because the expanded
+            # path was a directory that does not exist in self.pseudo_dirs. This
+            # is possible if you directly create files without making the
+            # directories first.
+            if not self.exists(p):
+                continue
+            self.rm_file(p)
+
     def size(self, path):
         """Size in bytes of the file at path"""
         if path not in self.store:
@@ -170,6 +193,7 @@ class MemoryFile(BytesIO):
     def __init__(self, fs=None, path=None, data=None):
         self.fs = fs
         self.path = path
+        self.created = datetime.utcnow().timestamp()
         if data:
             self.write(data)
             self.size = len(data)
