@@ -1,3 +1,4 @@
+import array
 from collections.abc import MutableMapping
 from .core import url_to_fs
 
@@ -55,8 +56,7 @@ class FSMap(MutableMapping):
             self.fs.rm(root + "/a")
 
     def clear(self):
-        """Remove all keys below root - empties out mapping
-        """
+        """Remove all keys below root - empties out mapping"""
         try:
             self.fs.rm(self.root, True)
             self.fs.mkdir(self.root)
@@ -106,7 +106,7 @@ class FSMap(MutableMapping):
         ----------
         values_dict: dict(str, bytes)
         """
-        values = {self._key_to_str(k): v for k, v in values_dict.items()}
+        values = {self._key_to_str(k): maybe_convert(v) for k, v in values_dict.items()}
         self.fs.pipe(values)
 
     def delitems(self, keys):
@@ -148,8 +148,7 @@ class FSMap(MutableMapping):
         """Store value in key"""
         key = self._key_to_str(key)
         self.fs.mkdirs(self.fs._parent(key), exist_ok=True)
-        with self.fs.open(key, "wb") as f:
-            f.write(value)
+        self.fs.pipe_file(key, maybe_convert(value))
 
     def __iter__(self):
         return (self._str_to_key(x) for x in self.fs.find(self.root))
@@ -169,15 +168,15 @@ class FSMap(MutableMapping):
         path = self._key_to_str(key)
         return self.fs.exists(path) and self.fs.isfile(path)
 
-    def __getstate__(self):
-        """Mapping should be pickleable"""
-        # TODO: replace with reduce to reinstantiate?
-        return self.fs, self.root
+    def __reduce__(self):
+        return FSMap, (self.root, self.fs, False, False, self.missing_exceptions)
 
-    def __setstate__(self, state):
-        fs, root = state
-        self.fs = fs
-        self.root = root
+
+def maybe_convert(value):
+    if isinstance(value, array.array) or hasattr(value, "__array__"):
+        # bytes-like things
+        value = bytearray(memoryview(value))
+    return value
 
 
 def get_mapper(url, check=False, create=False, missing_exceptions=None, **kwargs):
