@@ -97,11 +97,11 @@ known_implementations = {
     },
     "http": {
         "class": "fsspec.implementations.http.HTTPFileSystem",
-        "err": 'HTTPFileSystem requires "requests" to be installed',
+        "err": 'HTTPFileSystem requires "requests" and "aiohttp" to be installed',
     },
     "https": {
         "class": "fsspec.implementations.http.HTTPFileSystem",
-        "err": 'HTTPFileSystem requires "requests" to be installed',
+        "err": 'HTTPFileSystem requires "requests" and "aiohttp" to be installed',
     },
     "zip": {"class": "fsspec.implementations.zip.ZipFileSystem"},
     "gcs": {
@@ -112,6 +112,10 @@ known_implementations = {
         "class": "gcsfs.GCSFileSystem",
         "err": "Please install gcsfs to access Google Storage",
     },
+    "gdrive": {
+        "class": "gdrivefs.GoogleDriveFileSystem",
+        "err": "Please install gdrivefs for access to Google Drive",
+    },
     "sftp": {
         "class": "fsspec.implementations.sftp.SFTPFileSystem",
         "err": 'SFTPFileSystem requires "paramiko" to be installed',
@@ -138,6 +142,10 @@ known_implementations = {
         "class": "adlfs.AzureBlobFileSystem",
         "err": "Install adlfs to access Azure Datalake Gen2 and Azure Blob Storage",
     },
+    "az": {
+        "class": "adlfs.AzureBlobFileSystem",
+        "err": "Install adlfs to access Azure Datalake Gen2 and Azure Blob Storage",
+    },
     "cached": {"class": "fsspec.implementations.cached.CachingFileSystem"},
     "blockcache": {"class": "fsspec.implementations.cached.CachingFileSystem"},
     "filecache": {"class": "fsspec.implementations.cached.WholeFileCacheFileSystem"},
@@ -158,6 +166,19 @@ known_implementations = {
         "class": "fsspec.implementations.smb.SMBFileSystem",
         "err": 'SMB requires "smbprotocol" or "smbprotocol[kerberos]" installed',
     },
+    "jupyter": {
+        "class": "fsspec.implementations.jupyter.JupyterFileSystem",
+        "err": "Jupyter FS requires requests to be installed",
+    },
+    "jlab": {
+        "class": "fsspec.implementations.jupyter.JupyterFileSystem",
+        "err": "Jupyter FS requires requests to be installed",
+    },
+    "libarchive": {
+        "class": "fsspec.implementations.libarchive.LibArchiveFileSystem",
+        "err": "LibArchive requires to be installed",
+    },
+    "reference": {"class": "fsspec.implementations.reference.ReferenceFileSystem"},
 }
 
 minversions = {"s3fs": LooseVersion("0.3.0"), "gcsfs": LooseVersion("0.3.0")}
