@@ -10,6 +10,7 @@ from glob import has_magic
 from .dircache import DirCache
 from .transaction import Transaction
 from .utils import read_block, tokenize, stringify_path, other_paths
+from .config import apply_config
 
 logger = logging.getLogger("fsspec")
 
@@ -44,6 +45,7 @@ class _Cached(type):
         cls._pid = os.getpid()
 
     def __call__(cls, *args, **kwargs):
+        kwargs = apply_config(cls, kwargs)
         extra_tokens = tuple(
             getattr(cls, attr, None) for attr in cls._extra_tokenize_attributes
         )
@@ -131,6 +133,7 @@ class AbstractFileSystem(up, metaclass=_Cached):
         self._cached = True
         self._intrans = False
         self._transaction = None
+        self._invalidated_caches_in_transaction = []
         self.dircache = DirCache(**storage_options)
 
         if storage_options.pop("add_docs", None):
@@ -159,7 +162,7 @@ class AbstractFileSystem(up, metaclass=_Cached):
 
     @classmethod
     def _strip_protocol(cls, path):
-        """ Turn path from fully-qualified to file-system-specific
+        """Turn path from fully-qualified to file-system-specific
 
         May require FS-specific handling, e.g., for relative paths or links.
         """
@@ -191,7 +194,7 @@ class AbstractFileSystem(up, metaclass=_Cached):
 
     @classmethod
     def current(cls):
-        """ Return the most recently created FileSystem
+        """Return the most recently created FileSystem
 
         If no instance has been created, then create one with defaults
         """
@@ -221,6 +224,10 @@ class AbstractFileSystem(up, metaclass=_Cached):
         """Finish write transaction, non-context version"""
         self.transaction.complete()
         self._transaction = None
+        # The invalid cache must be cleared after the transcation is completed.
+        for path in self._invalidated_caches_in_transaction:
+            self.invalidate_cache(path)
+        self._invalidated_caches_in_transaction.clear()
 
     def invalidate_cache(self, path=None):
         """
@@ -232,7 +239,12 @@ class AbstractFileSystem(up, metaclass=_Cached):
             If None, clear all listings cached else listings at or under given
             path.
         """
-        pass  # not necessary to implement, may have no cache
+        # Not necessary to implement invalidation mechanism, may have no cache.
+        # But if have, you should call this method of parent class from your
+        # subclass to ensure expiring caches after transacations correctly.
+        # See the implementaion of FTPFileSystem in ftp.py
+        if self._intrans:
+            self._invalidated_caches_in_transaction.append(path)
 
     def mkdir(self, path, create_parents=True, **kwargs):
         """
@@ -320,12 +332,15 @@ class AbstractFileSystem(up, metaclass=_Cached):
         but contains nothing), None if not in cache.
         """
         parent = self._parent(path)
-        try:
+        if path.rstrip("/") in self.dircache:
             return self.dircache[path]
-        except KeyError:
-            pass
         try:
-            files = [f for f in self.dircache[parent] if f["name"] == path]
+            files = [
+                f
+                for f in self.dircache[parent]
+                if f["name"] == path
+                or (f["name"] == path.rstrip("/") and f["type"] == "directory")
+            ]
             if len(files) == 0:
                 # parent dir was listed but did not contain this file
                 raise FileNotFoundError(path)
@@ -334,7 +349,7 @@ class AbstractFileSystem(up, metaclass=_Cached):
             pass
 
     def walk(self, path, maxdepth=None, **kwargs):
-        """ Return all files belows path
+        """Return all files belows path
 
         List all files, recursing into subdirectories; output is iterator-style,
         like ``os.walk()``. For a simple list of files, ``find()`` is available.
@@ -495,10 +510,10 @@ class AbstractFileSystem(up, metaclass=_Cached):
         elif "/" in path[:ind]:
             ind2 = path[:ind].rindex("/")
             root = path[: ind2 + 1]
-            depth = 20 if "**" in path else path[ind2 + 1 :].count("/") + 1
+            depth = None if "**" in path else path[ind2 + 1 :].count("/") + 1
         else:
             root = ""
-            depth = 20 if "**" in path else 1
+            depth = None if "**" in path else path[ind + 1 :].count("/") + 1
 
         allpaths = self.find(root, maxdepth=depth, withdirs=True, detail=True, **kwargs)
         # Escape characters special to python regex, leaving our supported
@@ -537,10 +552,10 @@ class AbstractFileSystem(up, metaclass=_Cached):
         else:
             return list(out)
 
-    def exists(self, path):
+    def exists(self, path, **kwargs):
         """Is there a file at the given path"""
         try:
-            self.info(path)
+            self.info(path, **kwargs)
             return True
         except:  # noqa: E722
             # any exception allowed bar FileNotFoundError?
@@ -611,9 +626,15 @@ class AbstractFileSystem(up, metaclass=_Cached):
         except:  # noqa: E722
             return False
 
-    def cat_file(self, path):
+    def cat_file(self, path, start=None, end=None, **kwargs):
         """ Get the content of a file """
-        return self.open(path, "rb").read()
+        # explicitly set buffering off?
+        with self.open(path, "rb", **kwargs) as f:
+            if start is not None:
+                f.seek(start)
+            if end is not None:
+                return f.read(end - f.tell())
+            return f.read()
 
     def pipe_file(self, path, value, **kwargs):
         """Set the bytes of given file"""
@@ -756,18 +777,36 @@ class AbstractFileSystem(up, metaclass=_Cached):
     def cp_file(self, path1, path2, **kwargs):
         raise NotImplementedError
 
-    def copy(self, path1, path2, recursive=False, **kwargs):
-        """ Copy within two locations in the filesystem"""
+    def copy(self, path1, path2, recursive=False, on_error=None, **kwargs):
+        """Copy within two locations in the filesystem
+
+        on_error : "raise", "ignore"
+            If raise, any not-found exceptions will be raised; if ignore any
+            not-found exceptions will cause the path to be skipped; defaults to
+            raise unless recursive is true, where the default is ignore
+        """
+        if on_error is None and recursive:
+            on_error = "ignore"
+        elif on_error is None:
+            on_error = "raise"
+
         paths = self.expand_path(path1, recursive=recursive)
         path2 = other_paths(paths, path2)
         for p1, p2 in zip(paths, path2):
-            self.cp_file(p1, p2, **kwargs)
+            try:
+                self.cp_file(p1, p2, **kwargs)
+            except FileNotFoundError:
+                if on_error == "raise":
+                    raise
 
     def expand_path(self, path, recursive=False, maxdepth=None):
-        """Turn one or more globs or directories into a list of all matching files"""
+        """Turn one or more globs or directories into a list of all matching paths
+        to files or directories."""
         if isinstance(path, str):
             out = self.expand_path([path], recursive, maxdepth)
         else:
+            # reduce depth on each recursion level unless None or 0
+            maxdepth = maxdepth if not maxdepth else maxdepth - 1
             out = set()
             path = [self._strip_protocol(p) for p in path]
             for p in path:
@@ -775,10 +814,14 @@ class AbstractFileSystem(up, metaclass=_Cached):
                     bit = set(self.glob(p))
                     out |= bit
                     if recursive:
-                        out += self.expand_path(p)
+                        out |= set(
+                            self.expand_path(
+                                list(bit), recursive=recursive, maxdepth=maxdepth
+                            )
+                        )
                     continue
                 elif recursive:
-                    rec = set(self.find(p, withdirs=True))
+                    rec = set(self.find(p, maxdepth=maxdepth, withdirs=True))
                     out |= rec
                 if p not in out and (recursive is False or self.exists(p)):
                     # should only check once, for the root
@@ -824,7 +867,8 @@ class AbstractFileSystem(up, metaclass=_Cached):
     def _parent(cls, path):
         path = cls._strip_protocol(path.rstrip("/"))
         if "/" in path:
-            return cls.root_marker + path.rsplit("/", 1)[0]
+            parent = path.rsplit("/", 1)[0].lstrip(cls.root_marker)
+            return cls.root_marker + parent
         else:
             return cls.root_marker
 
@@ -896,7 +940,7 @@ class AbstractFileSystem(up, metaclass=_Cached):
             return f
 
     def touch(self, path, truncate=True, **kwargs):
-        """ Create empty file, or update timestamp
+        """Create empty file, or update timestamp
 
         Parameters
         ----------
@@ -917,7 +961,7 @@ class AbstractFileSystem(up, metaclass=_Cached):
         return sha256(str(self.info(path)).encode()).hexdigest()
 
     def read_block(self, fn, offset, length, delimiter=None):
-        """ Read a block of bytes from
+        """Read a block of bytes from
 
         Starting at ``offset`` of the file, read ``length`` bytes.  If
         ``delimiter`` is set then we ensure that the read starts and stops at
@@ -1259,7 +1303,7 @@ class AbstractBufferedFile(io.IOBase):
         return self.loc
 
     def seek(self, loc, whence=0):
-        """ Set current file location
+        """Set current file location
 
         Parameters
         ----------
@@ -1347,7 +1391,7 @@ class AbstractBufferedFile(io.IOBase):
             self.buffer = io.BytesIO()
 
     def _upload_chunk(self, final=False):
-        """ Write one part of a multi-block file upload
+        """Write one part of a multi-block file upload
 
         Parameters
         ==========
@@ -1459,7 +1503,7 @@ class AbstractBufferedFile(io.IOBase):
         return self.readinto(b)
 
     def close(self):
-        """ Close file
+        """Close file
 
         Finalizes writes, discards cache
         """
@@ -1490,7 +1534,8 @@ class AbstractBufferedFile(io.IOBase):
         return self.mode in {"wb", "ab"} and not self.closed
 
     def __del__(self):
-        self.close()
+        if not self.closed:
+            self.close()
 
     def __str__(self):
         return "<File-like object %s, %s>" % (type(self.fs).__name__, self.path)
