@@ -1,4 +1,4 @@
-from hashlib import md5
+from hashlib import sha256
 import math
 import os
 import pathlib
@@ -10,7 +10,7 @@ DEFAULT_BLOCK_SIZE = 5 * 2 ** 20
 
 
 def infer_storage_options(urlpath, inherit_storage_options=None):
-    """ Infer storage options from URL path and merge it with existing storage
+    """Infer storage options from URL path and merge it with existing storage
     options.
 
     Parameters
@@ -120,7 +120,7 @@ def infer_compression(filename):
 
 
 def build_name_function(max_int):
-    """ Returns a function that receives a single integer
+    """Returns a function that receives a single integer
     and returns it as a string padded by enough zero characters
     to align with maximum possible integer
 
@@ -198,7 +198,7 @@ def seek_delimiter(file, delimiter, blocksize):
 
 
 def read_block(f, offset, length, delimiter=None, split_before=False):
-    """ Read a block of bytes from a file
+    """Read a block of bytes from a file
 
     Parameters
     ----------
@@ -263,7 +263,7 @@ def read_block(f, offset, length, delimiter=None, split_before=False):
 
 
 def tokenize(*args, **kwargs):
-    """ Deterministic token
+    """Deterministic token
 
     (modified from dask.base)
 
@@ -275,11 +275,11 @@ def tokenize(*args, **kwargs):
     """
     if kwargs:
         args += (kwargs,)
-    return md5(str(args).encode()).hexdigest()
+    return sha256(str(args).encode()).hexdigest()
 
 
 def stringify_path(filepath):
-    """ Attempt to convert a path-like object to a string.
+    """Attempt to convert a path-like object to a string.
 
     Parameters
     ----------
@@ -359,3 +359,39 @@ def other_paths(paths, path2, is_dir=None):
     else:
         assert len(paths) == len(path2)
     return path2
+
+
+def is_exception(obj):
+    return isinstance(obj, BaseException)
+
+
+def get_protocol(url):
+    parts = re.split(r"(\:\:|\://)", url, 1)
+    if len(parts) > 1:
+        return parts[0]
+    return "file"
+
+
+def can_be_local(path):
+    """Can the given URL be used wih open_local?"""
+    from fsspec import get_filesystem_class
+
+    try:
+        return getattr(get_filesystem_class(get_protocol(path)), "local_file", False)
+    except (ValueError, ImportError):
+        # not in registry or import failed
+        return False
+
+
+def setup_logger(logname, level="DEBUG"):
+    import logging
+
+    logger = logging.getLogger(logname)
+    handle = logging.StreamHandler()
+    formatter = logging.Formatter(
+        "%(asctime)s - %(name)s - %(levelname)s " "- %(message)s"
+    )
+    handle.setFormatter(formatter)
+    logger.addHandler(handle)
+    logger.setLevel(level)
+    return logger
